/****************************************************************/
/* IBM Confidential OCO Source Materials                        */
/* AS854ZZ                                                      */
/* (C) Copyright IBM Corp. 2003                                 */
/*                                                              */
/* The source code for this program is not published or other-  */
/* wise divested of its trade secrets, irrespective of what has */
/* been deposited with the U.S. Copyright Office.               */
/****************************************************************/

#ifndef VXM_H
#define VXM_H

#include <stddef.h>
#include <time.h>
#include "vxmdefs.h"

#ifdef WIN32
	#ifdef VXM_EXPORTS
		#define VXM_DLLATTR __declspec(dllexport)
	#else
		#define VXM_DLLATTR __declspec(dllimport)
	#endif
#elif defined(TGTOS_SYMBIAN)
	#include <e32def.h>
	#ifdef VXM_EXPORTS
		#define VXM_DLLATTR   EXPORT_C
	#else
		#define VXM_DLLATTR   IMPORT_C
	#endif
#else
	#define VXM_DLLATTR
#endif

/** Return codes.
 */
enum VxmRc
{
	VXM_RC_OK,
	VXM_RC_MEM_ALLOC_ERROR,
	VXM_RC_INTERNAL_ERROR,
	VXM_RC_SEALED,
	VXM_RC_INVALID_STATE,
	VXM_RC_INVALID_LOCALE,
	VXM_RC_INSTALLATION_ERROR,
	VXM_RC_PARSE_FAILED,
	VXM_RC_NO_DOCUMENT,
	VXM_RC_HANDLER_NOT_FOUND,
	VXM_RC_NO_URI_RESOLVER,
	VXM_RC_SEMANTIC_ERROR,
	VXM_RC_ECMASCRIPT_ERROR,
	VXM_RC_CANCELED,
	VXM_RC_INVALID_LISTENING_MODE,
	VXM_RC_STATE_UNREACHABLE,
	VXM_RC_INVALID_REC_HANDLE,
	VXM_RC_NO_SCOPE_FACTORY,
	VXM_RC_NOT_RUNNING,
	VXM_RC_INVALID_VALUE,
	VXM_NO_SYNC_FIELD,

	VXM_SCOPE_RC_NO_OBJECT
};

/** Listening modes.
 */
enum VxmListeningMode
{
	VXM_LISTENING_MODE_PTT,  // push-to-talk
	VXM_LISTENING_MODE_PTA,  // push-to-activate
	VXM_LISTENING_MODE_AL,   // always listening
	VXM_LISTENING_MODE_APTA, // automatic push-to-activate
};

/** Log levels.
 */
enum VxmLogLevel
{
	VXM_LOG_LEVEL_VERBOSE,
	VXM_LOG_LEVEL_INFO,
	VXM_LOG_LEVEL_WARNING,
	VXM_LOG_LEVEL_SEVERE
};

/** Voice parameters.
 * The valid value of VXM_VOICE_PARAMETER_ASR_REJECTION_THRESHOLD are 0-10 (2 is default value) 
 * The valid value of VXM_VOICE_PARAMETER_TTS_GENDER are male (0; default) and female (1).
 * The valid value of VXM_VOICE_PARAMETER_TTS_PITCH are 
 * very low (0), low (1), medium (2; default), high (3) and very high (4).
 * The valid value of VXM_VOICE_PARAMETER_TTS_SPEED are 
 * very low (0), low (1), medium (2; default), high (3) and very high (4).
 * The valid value of VXM_VOICE_PARAMETER_TTS_VOLUME are
 * silent (0), very soft (1), soft (2), medium (3; default), loud (4) and very loud (5).  
 * The valid value of VXM_VOICE_PARAMETER_TTS_CTTS are
 * disable (0; default), enable (1)
 */
enum VxmVoiceParameter
{
	VXM_VOICE_PARAMETER_ASR_REJECTION_THRESHOLD,
	VXM_VOICE_PARAMETER_TTS_GENDER,
	VXM_VOICE_PARAMETER_TTS_PITCH,
	VXM_VOICE_PARAMETER_TTS_SPEED,
	VXM_VOICE_PARAMETER_TTS_VOLUME,
	VXM_VOICE_PARAMETER_TTS_CTTS
};

/** Class VxmFeatureDescriptor is an abstract base class that represents a set
 *  of configurable features that allow the interpreter to be tuned for
 *  different application environments. Most visual browsers will find the
 *  feature defaults to be more than suitable for their environment. Otherwise,
 *  visual browsers can create a VxmFeatureDescriptor instance using the static
 *  createInstance factory method, call its set methods, and then call
 *  VxmInterpreter::setFeatureDescriptor. Visual browsers must delete this
 *  instance when finished with it. An alternative is to call
 *  VxmInterpreter::getFeatureDescriptor to obtain the default feature
 *  descriptor, and then call its set methods. Some feature values may be
 *  sealed in specific contexts, such as while an interpreter is in the
 *  PROCESSING sub-state.
 */
class VXM_DLLATTR VxmFeatureDescriptor
{
public:
	/** Static factory method to construct a VxmFeatureDescriptor instance.
	 *  Visual browsers must delete the instance when finished with it.
	 *   @return A new VxmFeatureDescriptor instance, or NULL on failure.
	 */
	static VxmFeatureDescriptor* createInstance();

	/** Virtual destructor to ensure that the appropriate destructor is called
	 *  when a VxmFeatureDescriptor instance is deleted.
	 */
	virtual ~VxmFeatureDescriptor() {}

	/** Set the validation mode. The validation mode is true by default.
	 *   @param bValidation (in) The validation mode. True means attempt to
	 *    validate the document; false means no validation.
	 */
	virtual void setValidation(bool bValidation) = 0;

	/** Get the validation mode.
	 *   @return The validation mode.
	 */
	virtual bool getValidation() = 0;

	/** Set the caching flag. Caching is enabled by default. The caching flag is
	 *  sealed while the VxmInterpreter is in the PROCESSING sub-state, in which
	 *  case this method will return an error.
	 *   @param bEnabled (in) True to enable caching, or false to disable
	 *    caching.
	 *   @return VXM_RC_OK, VXM_RC_SEALED.
	 */
	virtual VxmRc setCaching(bool bEnabled) = 0;

	/** Get the caching flag.
	 *   @return True if caching is enabled, or false if caching is disabled.
	 */
	virtual bool getCaching() = 0;
};

/** Class VxmPropertyDescriptor is an abstract base class that represents a set
 *  of configurable properties that allow the interpreter to be tuned for
 *  different application environments. Most visual browsers will find the
 *  property defaults to be more than suitable for their environment. Otherwise,
 *  visual browsers can create a VxmPropertyDescriptor instance using the static
 *  createInstance factory method, call its set methods, and then call
 *  VxmInterpreter::setPropertyDescriptor. Visual browsers must delete this
 *  instance when finished with it. An alternative is to call
 *  VxmInterpreter::getPropertyDescriptor to obtain the default property
 *  descriptor, and then call its set methods. Some property values may be
 *  sealed in specific contexts, such as while an interpreter is in the
 *  PROCESSING sub-state.
 */
class VXM_DLLATTR VxmPropertyDescriptor
{
public:
	/** Static factory method to construct a VxmPropertyDescriptor instance.
	 *  Visual browsers must delete the instance when finished with it.
	 *   @return A new VxmPropertyDescriptor instance, or NULL on failure.
	 */
	static VxmPropertyDescriptor* createInstance();

	/** Virtual destructor to ensure that the appropriate destructor is called
	 *  when a VxmPropertyDescriptor instance is deleted.
	 */
	virtual ~VxmPropertyDescriptor() {}

	/** Set the locale of the VoiceXML interpreter. The locale is US English by
	 *  default. The locale can only be set while the VxmInterpreter is in the
	 *  DEALLOCATED state, otherwise it is sealed, and this method will return
	 *  an error.
	 *   @param pstrLocale (in) Pointer to the 5-character "language-country"
	 *    code, based on ISO639 and using "-" as the separator, i.e. "en-US".
	 *   @return VXM_RC_OK, VXM_RC_SEALED.
	 */
	virtual VxmRc setLocale(const wchar_t *pstrLocale) = 0;

	/** Get the locale of the VoiceXML interpreter.
	 *   @return Pointer to the 5-character "language_country" code.
	 */
	virtual const wchar_t* getLocale() = 0;

	/** Set the listening mode.	The listening mode can only be set while the
	 *  VxmInterpreter is in the DEALLOCATED state, otherwise it is sealed, and
	 *  this method will return an error.
	 *   @param listeningMode (in) The listening mode.
	 *   @return VXM_RC_OK, VXM_RC_SEALED.
	 */
	virtual VxmRc setListeningMode(VxmListeningMode listeningMode) = 0;

	/** Get the listening mode.
	 *   @return The listening mode.
	 */
	virtual VxmListeningMode getListeningMode() = 0;

	/** Set the cache size. The cache size can only be set while the
	 *  VxmInterpreter is in the DEALLOCATED state, otherwise it is sealed, and
	 *  this method will return an error.
	 *   @param iCacheSize (in) The cache size, in kilobytes.
	 *   @return VXM_RC_OK, VXM_RC_SEALED.
	 */
	virtual VxmRc setCacheSize(int iCacheSize) = 0;

	/** Get the cache size.
	 *   @return The cache size, in kilobytes.
	 */
	virtual int getCacheSize() = 0;

	/** Query the VoiceXML interpreter for support for a locale.
	 *   @param pstrLocale (in) Pointer to the 5-character "language-country"
	 *    code, based on ISO639 and using "-" as the separator, i.e. "en-US".
	 *   @return true or false.
	 */
	virtual bool supportedLocale(const wchar_t *pstrLocale) = 0;

	/** Query the VoiceXML interpreter to determine if this locate will require
	 *   a deallocate() -> setLocale() -> allocate() sequence, or if the interpreter
	 *   can handle this locale in its current state.
	 *   @param pstrLocale (in) Pointer to the 5-character "language-country"
	 *    code, based on ISO639 and using "-" as the separator, i.e. "en-US".
	 *   @return true or false.
	 */
	virtual bool localeSwitchRequired(const wchar_t *pstrLocale) = 0;

	/** Set the install path of the vxm library. This is the directory where
	 *  the vxm library is located. The install path can only be set while the 
	 *  VxmInterpreter is in the DEALLOCATED state, otherwise it is sealed, and 
	 *  this method will return an error.
	 *   @param pstrPath (in) Pointer to the full path of the install directory.
	 *   @return VXM_RC_OK, VXM_RC_SEALED.
	 */
	virtual VxmRc setInstallPath(const wchar_t *pstrPath) = 0;

	/** Get the install path of the vxm library.
	 *   @return Pointer to the full path of the install directory.
	 */
	virtual const wchar_t* getInstallPath() = 0;

	/** Set the resource path of the vxm library. This is the directory where
	 *  the external resources (localized strings, tts binaries, etc.) for the vxm
	 *  library are located. On some platforms this will be the same as the 
	 *  install directory. The resource path can only be set while the 
	 *  VxmInterpreter is in the DEALLOCATED state, otherwise it is sealed, and 
	 *  this method will return an error.
	 *   @param pstrPath (in) Pointer to the full path of the resource directory.
	 *   @return VXM_RC_OK, VXM_RC_SEALED.
	 */
	virtual VxmRc setResourcePath(const wchar_t *pstrPath) = 0;

	/** Get the resource path of the vxm library.
	 *   @return Pointer to the full path of the resource directory.
	 */
	virtual const wchar_t* getResourcePath() = 0;

	/** Set the property value. The property value can only be set while the
	 *  VxmInterpreter is in the DEALLOCATED state, otherwise it is sealed, and
	 *  this method will return an error.
	 *   @param name (in) The property name.
	 *   @param value (in) The property value.
	 *   @return VXM_RC_OK, VXM_RC_SEALED, VXM_RC_INVALID_VALUE
	 */
	virtual VxmRc setProperty(VxmVoiceParameter name, int value) = 0;

	/** Get the property value. 
	 *   @param name (in) The property name.
	 *   @param value (out) The property value.
	 *   @return VXM_RC_OK, VXM_RC_INVALID_VALUE
	 */
	virtual VxmRc getProperty(VxmVoiceParameter name, int *value) = 0;

	/** Query the supported property value.
	 *   The valid values of voice parameter highly depend on a languages type as well as TTSs (i.e. formant or concatenative
	 *   synthesis). The value returns from this function based on current setting locale and bCTTS value.  
	 *   @param name (in) The property name.
	 *   @param value (out)  Pointer to the array of supported property values.
	 *   @param arraySize (out) Pointer to the number of elements in the valid property value array.
	 *     if it is 0, the property is not supported.    
	 *   @param bCTTS (in) Set to true if browser is using concatenative TTS.
	 *   @return VXM_RC_OK, VXM_RC_INVALID_VALUE
	 **/
	virtual VxmRc querySupportedPropertyValue(VxmVoiceParameter name, int** value, int *arraySize, bool bCTTS = false) = 0;
};

/** Class VxmXmlEventListener is an abstract base class that provides a listener
 *  interface for XML events created by a VxmInterpreter instance.
 */
class VxmXmlEventListener
{
public:
	/** Process an XML event.
	 *   @param pstrElementType (in) Pointer to the type of element that sent this event.
	 *          The value will be either "form" or "link".
	 *   @param pstrElementId (in) Pointer to the Id of the element that sent this event.
	 *   @param pstrEvent (in) Pointer to the name of this event.
	 */
	virtual void vxmXmlEvent(const wchar_t *pstrElementType, const wchar_t *pstrElementId, const wchar_t *pstrEvent) = 0;
};

/** Class VxmStateListener is an abstract base class that provides a listener
 *  interface for VxmInterpreter state changes and asynchronous errors.
 */
class VxmStateListener
{
public:
	/** Process a VxmInterpreter state change.
	 *   @param lPrevState The previous VxmInterpreter state.
	 *   @param lNewState The new VxmInterpreter state.
	 */
	virtual void vxmStateChange(long lPrevState, long lNewState) = 0;

	/** Process a VxmInterpreter asynchronous error.
	 *   @param rc The error number.
	 */
	virtual void vxmError(VxmRc rc) = 0;
};

/** Class VxmLogListener is an abstract base class that provides a listener
 *  interface for VxmInterpreter log entries.
 */
class VxmLogListener
{
public:
	/** Process a VxmInterpreter log entry.
	 *   @param pstrLogMessage Pointer to the message to be logged.
	 *   @param logLevel The log level of the message.
	 *   @param secs System time at which the logged event occurred, in seconds.
	 *   @param iMillisecs Number of milliseconds past "secs" at which the
	 *    logged event occurred.
	 */
	virtual void vxmLogEntry(const wchar_t *pstrLogMessage, VxmLogLevel logLevel, time_t secs, int iMillisecs) = 0;
};

class VxmScopeFactory;

/** VxmContext is an opaque handle to an implementation's ECMAScript context.
 */
typedef void * VxmContext;

/** GlobalScope is an opaque handle to an implementation's global ECMAScript scope.
 */
typedef void * GlobalScope;

/** VxmScope forward declared see vxmscope.h
 */
class VxmScope;

VxmScope * createScope(VxmContext context, GlobalScope globalScope);
typedef VxmScope * (CreateScopeCB)(VxmContext context, GlobalScope globalScope);

/** Class VxmInterpreter is an abstract base class that represents a serially
 *  reusable VoiceXML interpreter. Visual browsers can create a VxmInterpreter
 *  instance using the static createInstance factory method, set its features
 *  and properties, then have it process X+V voice handlers. The VxmInterpreter
 *  has several states, sub-states, and listening states. Each transition is
 *  sent to all registered VxmStateListener instances. Optionally, visual
 *  browsers may use the getState method to query the current state. All state
 *  change events are sent asynchronously to the visual browser. Upon creation,
 *  the interpreter transitions to the DEALLOCATED state to signify that no
 *  interpreter resources have been allocated. Furthermore, the interpreter
 *  enters the STOPPED sub-state to signify that no handler processing is
 *  occurring, and the LISTENING_NOT_AVAILABLE listening state to signify that
 *  the interpreter is not ready to start listening. Once in the DEALLOCATED
 *  state, visual browsers can set the features and properties to be used with
 *  this interpreter session. After setting the features and properties, visual
 *  browsers can call the allocate method to bind all system resources to the
 *  interpreter. During the allocation process, the interpreter will enter the
 *  ALLOCATING state. If the allocation process completes normally, the
 *  interpreter will enter the ALLOCATED state, otherwise it will return to the
 *  DEALLOCATED state. Once in the ALLOCATED state, visual browsers can add X+V
 *  voice handlers by calling the addHandler method, and set the URI resolver by
 *  calling the setUriResolver method. After the handlers have been added,
 *  visual browsers can call the runHandler method to process a handler. The
 *  runHandler method can only be called in the ALLOCATED state. When the
 *  runHandler method is called, the interpreter transitions to the PROCESSING
 *  sub-state to signify the interpreter's handler processing phase. When the
 *  interpreter is ready to start listening, it transitions to the
 *  LISTENING_STOPPED listening state. Then when the user is ready to begin
 *  speaking, the startListening method should be called, causing the
 *  interpreter to transition to the LISTENING_STARTED listening state. If the
 *  listening mode is push-to-activate, the interpreter returns to the
 *  LISTENING_NOT_AVAILABLE listening state when silence is detected. If the
 *  listening mode is push-to-talk, then when the user is finished speaking, the
 *  stopListening method should be called, causing the interpreter to return to
 *  the LISTENING_NOT_AVAILABLE listening state. Visual browsers can run another
 *  handler by either waiting for the runHandler method to complete (the
 *  interpreter's sub-state will transition back to STOPPED) or by calling the
 *  cancel method on the interpreter in the PROCESSING sub-state. The runHandler
 *  method is synchronous and may not complete for many seconds. Visual browsers
 *  can call the runHandler method using a separate thread. When cancel is
 *  called, the interpreter enters the STOPPING sub-state until the current
 *  handler processing stops, at which point the interpreter enters the STOPPED
 *  sub-state. When visual browsers no longer need interpreter services, they
 *  may call the deallocate method to unbind all resources associated with the
 *  interpreter. During this process, the interpreter transitions to the
 *  DEALLOCATING state to signal the start of deallocation. Once the interpreter
 *  completes the process, it transitions back to the DEALLOCATED state. If an
 *  interpreter is in the PROCESSING sub-state when deallocated is called, the
 *  cancel method is first called before beginning the deallocation process.
 *  Visual browsers must delete the VxmInterpreter instance when finished with
 *  it.
 */
class VXM_DLLATTR VxmInterpreter
{
public:
	/** Bit of state that is set when the interpreter is in the deallocated
	 *  state. A DEALLOCATED interpreter does not have the resources necessary
	 *  to carry out its basic functions. In the DEALLOCATED state, many of the
	 *  methods of the interpreter will return an error when called. A
	 *  DEALLOCATED interpreter can transition to the ALLOCATED state via the
	 *  ALLOCATING state following a call to the allocate method. An interpreter
	 *  returns to the DEALLOCATED state via the DEALLOCATING state with a call
	 *  to the deallocate method. An interpreter is always created in the
	 *  DEALLOCATED state. The DEALLOCATED state has just one sub-state, namely
	 *  STOPPED. 
	 */
	static const long DEALLOCATED;

	/** Bit of state that is set when an interpreter is being allocated. This is
	 *  the transition state between DEALLOCATED to ALLOCATED following a call
	 *  to the allocate method. In the ALLOCATING state, most interpreter
	 *  methods will return an error. The ALLOCATING state has just one sub-
	 *  state, namely STOPPED.
	 */
	static const long ALLOCATING;

	/** Bit of state that is set when the interpreter is in the allocated state.
	 *  An ALLOCATED interpreter has acquired all the resources necessary to
	 *  carry out its core functions. An ALLOCATED interpreter can transition to
	 *  the DEALLOCATED state via the DEALLOCATING state following a call to the
	 *  deallocate method. An interpreter returns to the ALLOCATED state via the
	 *  ALLOCATING state with a call to the allocate method. The ALLOCATED state
	 *  has sub-states for STOPPED, PROCESSING, and STOPPING.
	 */
	static const long ALLOCATED;

	/** Bit of state that is set when the interpreter is being deallocated. This
	 *  is the transition state between ALLOCATED and DEALLOCATED following a
	 *  call to the deallocate method. In the DEALLOCATING state, most
	 *  interpreter methods will return an error. The DEALLOCATING state has
	 *  just one sub-state, namely STOPPED.
	 */
	static const long DEALLOCATING;

	/** Bit of state that is set when the interpreter is in the stopped sub-
	 *  state. An interpreter in the STOPPED sub-state is not processing any
	 *  document and is ready to process a document. A STOPPED interpreter can
	 *  transition to the PROCESSING sub-state following a call to the run
	 *  method. An interpreter returns to the STOPPED sub-state via the STOPPING
	 *  sub-state following a call to the cancel method. An interpreter is
	 *  always created in the STOPPED sub-state.
	 */
	static const long STOPPED;

	/** Bit of state that is set when the interpreter is in the processing
	 *  sub-state. An interpreter in the PROCESSING sub-state is actively
	 *  processing a document. The PROCESSING interpreter can transition to the
	 *  STOPPED sub-state via the STOPPING sub-state following a call to the
	 *  cancel method. An interpreter returns to the PROCESSING sub-state
	 *  following a call to the run method.
	 */
	static const long PROCESSING;

	/** Bit of state that is set when the interpreter is in the stopping sub-
	 *  state. An interpreter in the STOPPING sub-state is actively attempting
	 *  to abort the current document processing. This is a transition sub-state
	 *  between PROCESSING and STOPPED following a call to the cancel method.
	 */
	static const long STOPPING;

	/** Bit of state that is set when the interpreter is listening for speech
	 *  input from the user. The LISTENING_STOPPED interpreter can transition to
	 *  LISTENING_STARTED following a call to the startListening method. An
	 *  interpreter transitions to LISTENING_NOT_AVAILABLE following a call to
	 *  the stopListening method if the listening mode is push-to-talk, or
	 *  following silence if the listening mode is push-to-activate.
	 */
	static const long LISTENING_STARTED;

	/** Bit of state that is set when the interpreter is not listening for
	 *  speech input from the user, but is ready to start listening. The
	 *  LISTENING_NOT_AVAILABLE interpreter transitions to LISTENING_STOPPED
	 *  when it is ready to start listening. An interpreter transitions to
	 *  LISTENING_STARTED following a call to the startListening method.
	 */
	static const long LISTENING_STOPPED;

	/** Bit of state that is set when the interpreter is not listening for
	 *  speech input from the user, and is not ready to start listening. The
	 *  LISTENING_STARTED interpreter can transition to LISTENING_NOT_AVAILABLE
	 *  following a call to the stopListening method if the listening mode is
	 *  push-to-talk, or following silence if the listening mode is push-to-
	 *  activate. An interpreter transitions to LISTENING_STOPPED when it is
	 *  ready to start listening.
	 */
	static const long LISTENING_NOT_AVAILABLE;

	/** Static factory method to construct a VxmInterpreter instance. An
	 *  interpreter is created in the DEALLOCATED state with the STOPPED sub-
	 *  state. Visual browsers must delete the instance when finished with it.
	 *   @return A new VxmInterpreter instance, or NULL on failure.
	 */
	static VxmInterpreter* createInstance();

	/** Virtual destructor to ensure that the appropriate destructor is called
	 *  when a VxmInterpreter instance is deleted.
	 */
	virtual ~VxmInterpreter() {}

	/** Set the name of this interpreter session. The name can be used to
	 *  uniquely identify a particular interpreter instance. The name is used
	 *  during logging.
	 *   @param pstrName (in) Pointer to the name to give the interpreter.
	 */
	virtual void setName(const wchar_t *pstrName) = 0;

	/** Retrieve the interpreter name.
	 *   @return Pointer to the interpreter name, or NULL if no name has been
	 *    assigned.
	 */
	virtual const wchar_t* getName() = 0;

	/** Set the site document, which represents the command-and-control or
	 *  global customizations. This is used by all VxmInterpreter instances,
	 *  during the run method. An error is returned if any VxmInterpreter
	 *  instance is in the PROCESSING sub-state.
	 *   @param pstrSiteDocument (in) Pointer to the site document.
	 *   @return VXM_RC_OK, VXM_RC_INVALID_STATE.
	 */
	static VxmRc setSiteDocument(const wchar_t *pstrSiteDocument);

	/** Get the site document.
	 *   @return Pointer to the site document.
	 */
	static const wchar_t* getSiteDocument();

	/** Set the feature descriptor. Visual browsers should set the feature
	 *  descriptor only in the DEALLOCATED state. An error is returned if the
	 *  feature descriptor is set in the ALLOCATED, ALLOCATING, or DEALLOCATING
	 *  states.
	 *   @param pFd (in) Pointer to the features to enable for this interpreter
	 *    session.
	 *   @return VXM_RC_OK, VXM_RC_INVALID_STATE.
	 */
	virtual VxmRc setFeatureDescriptor(VxmFeatureDescriptor *pFd) = 0;

	/** Return the feature descriptor for the current interpreter session.
	 *   @return The current feature descriptor.
	 */
	virtual VxmFeatureDescriptor* getFeatureDescriptor() = 0;

	/** Set the property descriptor. Visual browsers should set the property
	 *  descriptor only in the DEALLOCATED state. An error is returned if the
	 *  property descriptor is set in the ALLOCATED, ALLOCATING, or DEALLOCATING
	 *  states.
	 *   @param pPd (in) Pointer to the properties to enable for this
	 *    interpreter session.
	 *   @return VXM_RC_OK, VXM_RC_INVALID_STATE.
	 */
	virtual VxmRc setPropertyDescriptor(VxmPropertyDescriptor *pPd) = 0;

	/** Return the property descriptor for the current interpreter session.
	 *   @return The current property descriptor.
	 */
	virtual VxmPropertyDescriptor* getPropertyDescriptor() = 0;

	/** Allocate the resources required for the interpreter and put it into the
	 *  ALLOCATED state. When this method returns successfully, the ALLOCATED
	 *  bit of the interpreter state is set. During the processing of the
	 *  method, the interpreter is temporarily in the ALLOCATING state. When the
	 *  interpreter reaches the ALLOCATED state, the sub-state is STOPPED. While
	 *  this method is being processed, state changes are sent to any registered
	 *  VxmListenerState instances. The allocate method should be called for an
	 *  interpreter in the DEALLOCATED state. The method has no effect for an
	 *  interpreter in either the ALLOCATED, ALLOCATING, or DEALLOCATING states.
	 *  If any problems are encountered during the allocation process so that
	 *  the interpreter cannot be allocated, the interpreter returns to the
	 *  DEALLOCATED state and an error is returned. Allocating the resources for
	 *  the interpreter may be fast (less than a second) or slow (several 10s of
	 *  seconds) depending on a range of factors. Since the allocate method does
	 *  not return until allocation is completed, visual browsers may want to
	 *  perform allocation on a background thread and proceed with other
	 *  activities.
	 *   @return VXM_RC_OK, VXM_RC_MEM_ALLOC_ERROR, VXM_RC_INTERNAL_ERROR,
	 *    VXM_RC_INVALID_STATE, VXM_RC_INVALID_LOCALE,
	 *    VXM_RC_INSTALLATION_ERROR, VXM_RC_INVALID_LISTENING_MODE.
	 */
	virtual VxmRc allocate() = 0;

	/** Free the resources of the interpreter that were acquired during
	 *  allocation and normal operation. A successful call to deallocate returns
	 *  the interpreter to the DEALLOCATED state. A deallocated interpreter can
	 *  be restarted with a subsequent call to allocate. While this method is
	 *  being processed, state changes are sent to any registered
	 *  VxmListenerState instances. The deallocate method should only be called
	 *  for an interpreter in the ALLOCATED state. This method has no effect for
	 *  an interpreter in either the DEALLOCATED, DEALLOCATING, or ALLOCATING
	 *  states. Deallocating resources for an engine is not always immediate.
	 *  Since the deallocate method does not return until complete, visual
	 *  browsers may want to perform deallocation in a separate thread.
	 *   @return VXM_RC_OK, VXM_RC_INVALID_STATE.
	 */
	virtual VxmRc deallocate() = 0;

	/** Set the VoiceXML document for this interpreter session. This document
	 *  will be the document the interpreter processes when the run method is
	 *  called. The URI and timestamp parameters uniquely identify the document,
	 *  however the specific format is up to the caller. That is, if setDocument
	 *  is later called again with the same URI and timestamp, the interpreter
	 *  may assume that the text is the same as well. That way, the interpreter
	 *  may use a cached version of the parse tree, reco grammars, etc.
	 *  Typically, the URI is the location of the VoiceXML document, and the
	 *  timestamp is the time that the VoiceXML document was fetched. The
	 *  document may only be set when the interpreter is in the STOPPED sub-
	 *  state. The document may be reset when the interpreter returns to the
	 *  STOPPED sub-state after being in the PROCESSING sub-state.
	 *   @param pstrDocText (in) Pointer to the document text.
	 *   @param pstrDocUri (in) Pointer to the URI for the document.
	 *   @param lDocTimeStamp (in) The document timestamp. Format is not
	 *    important, as long as the timestamp changes each time the document is
	 *    refreshed.
	 *   @return VXM_RC_OK, VXM_RC_MEM_ALLOC_ERROR, VXM_RC_INTERNAL_ERROR,
	 *    VXM_RC_INVALID_STATE, VXM_PARSE_FAILED.
	 */
	virtual VxmRc setDocument(const wchar_t *pstrDocText, const wchar_t *pstrDocUri, long lDocTimeStamp) = 0;

	/** Return the document text.
	 *   @return Pointer to the document text.
	 */
	virtual const wchar_t* getDocument() = 0;

	/** Set the ECMAScript context. The context may only be set when the
	 *  interpreter is in the STOPPED sub-state.
	 *   @param context (in) The ECMAScript context.
	 *   @return VXM_RC_OK, VXM_RC_INVALID_STATE.
	 */
	virtual VxmRc setECMAScriptContext(VxmContext context) = 0;

	/** Get the ECMAScript context.
	 *   @return The ECMAScript context.
	 */
	virtual VxmContext getECMAScriptContext() = 0;

	/** Set the ECMAScript scope. The scope may only be set when the interpreter
	 *  is in the STOPPED sub-state.
	 *   @param scope (in) The ECMAScript scope.
	 *   @return VXM_RC_OK, VXM_RC_INVALID_STATE.
	 */
	virtual VxmRc setECMAScriptScope(GlobalScope scope) = 0;

	/** Get the ECMAScript scope.
	 *   @return The ECMAScript scope.
	 */
	virtual GlobalScope getECMAScriptScope() = 0;

	/** Add an X+V voice handler to the list of handlers for this interpreter
	 *  session. The handler can be executed by calling the runHandler method.
	 *  The URI and timestamp parameters uniquely identify the document
	 *  containing the voice handler, however the specific format is up to the
	 *  visual browser. That is, if addHandler is later called again with a
	 *  handler that has the same "id" attribute, and the same URI and timestamp
	 *  parameters are specified, the interpreter may assume that the handler
	 *  text is the same as well. That way, the interpreter may use a cached
	 *  version  of the parse tree, reco grammars, etc. Typically, the URI is
	 *  the location of the document, and the timestamp is the time that the
	 *  document was fetched. A handler may only be added when the interpreter
	 *  is in the STOPPED sub-state.
	 *   @param pstrHandlerText (in) Pointer to the text of an X+V voice handler,
	 *    consisting of a VoiceXML <form> element and its contents.
	 *   @param pstrDocUri (in) Pointer to the URI for the document containing
	 *    the handler.
	 *   @param lDocTimeStamp (in) Timestamp is used with a caching policy where 
	 *    the oldest handlers are removed from the cache, and new handlers replace
	 *    older versions.  Format is not important.  However, timestamp should 
	 *    represent the date and time when the document was fetched over the network,
	 *    and should change each time the document is refreshed.
	 *   @param pstrBaseUri (in) Pointer to the base URI for resolving external
	 *    references encountered during the execution of this handler. Unless
	 *    specified by the markup, pstrBaseUri should be the same as pstrUri.
	 *   @param pstrHandlerId (in) Pointer to the X+V voice handler ID.   
	 *   @return VXM_RC_OK, VXM_RC_MEM_ALLOC_ERROR, VXM_RC_INTERNAL_ERROR,
	 *    VXM_RC_INVALID_STATE, VXM_PARSE_FAILED.
	 */
	virtual VxmRc addHandler(const wchar_t* pstrHandlerText, const wchar_t* pstrDocURI, long lDocTimeStamp, const wchar_t* pstrBaseUri, const wchar_t* pstrHandlerId) = 0;

	/** Clear the list of handlers for this interpreter session. The handlers
	 *  may only be cleared when the interpreter is in the STOPPED sub-state.
	 *   @return VXM_RC_OK, VXM_RC_MEM_ALLOC_ERROR, VXM_RC_INTERNAL_ERROR,
	 *    VXM_RC_INVALID_STATE.
	 */
	virtual VxmRc clearHandlers() = 0;

	/** Set the current URI resolver, which will be used by the interpreter to
	 *  resolve URIs. The URI resolver may only be set when the interpreter is
	 *  in the STOPPED sub-state. The URI resolver may be reset when the
	 *  interpreter returns to the STOPPED sub-state after being in the
	 *  PROCESSING sub-state.
	 *   @param pUriResolver (in) Pointer to the URI resolver to be used.
	 *   @return VXM_RC_OK, VXM_RC_INVALID_STATE.
	 */
	virtual VxmRc setUriResolver(VxmUriResolver *pUriResolver) = 0;

	/** Get the current URI resolver.
	 *   @return Pointer to the current URI resolver, or NULL if there is none.
	 */
	virtual VxmUriResolver* getUriResolver() = 0;

	/** Add an XML event listener to the list of registered listeners. An XML
	 *  event listener may only be added when the interpreter is in the STOPPED
	 *  sub-state.
	 *   @param pXmlEventListener (in) Pointer to the listener to add.
	 *   @return VXM_RC_OK, VXM_RC_MEM_ALLOC_ERROR, VXM_RC_INTERNAL_ERROR,
	 *    VXM_RC_INVALID_STATE.
	 */
	virtual VxmRc addXMLEventListener(VxmXmlEventListener *pXmlEventListener) = 0;

	/** Remove an XML event listener from the list of previously registered
	 *  listeners. It is not an error to remove a listener that was never
	 *  registered. An XML event listener may only be removed when the
	 *  interpreter is in the STOPPED sub-state.
	 *   @param pXmlEventListener (in) Pointer to the listener to de-register.
	 *   @return VXM_RC_OK, VXM_RC_INVALID_STATE.
	 */
	virtual VxmRc removeXMLEventListener(VxmXmlEventListener *pXmlEventListener) = 0;

	/** Add a state listener to the list of registered listeners. A state
	 *  listener may only be added when the interpreter is in the STOPPED sub-
	 *  state.
	 *   @param pStateListener (in) Pointer to the listener to add.
	 *   @return VXM_RC_OK, VXM_RC_MEM_ALLOC_ERROR, VXM_RC_INTERNAL_ERROR,
	 *    VXM_RC_INVALID_STATE.
	 */
	virtual VxmRc addStateListener(VxmStateListener *pStateListener) = 0;

	/** Remove a state listener from the list of previously registered
	 *  listeners. It is not an error to remove a listener that was never
	 *  registered. A state listener may only be removed when the interpreter is
	 *  in the STOPPED sub-state.
	 *   @param pStateListener (in) Pointer to the listener to de-register.
	 *   @return VXM_RC_OK, VXM_RC_INVALID_STATE.
	 */
	virtual VxmRc removeStateListener(VxmStateListener *pStateListener) = 0; 

	/** Add a log listener to the list of registered listeners. A log listener
	 *  may only be added when the interpreter is in the STOPPED sub-state.
	 *   @param pLogListener (in) Pointer to the listener to add.
	 *   @param logLevel (in) Level of logging desired. All log entries this
	 *    level or higher will be sent to this VxmLogListener.
	 *   @return VXM_RC_OK, VXM_RC_MEM_ALLOC_ERROR, VXM_RC_INTERNAL_ERROR,
	 *    VXM_RC_INVALID_STATE.
	 */
	virtual VxmRc addLogListener(VxmLogListener *pLogListener, VxmLogLevel logLevel) = 0;

	/** Remove a log listener from the list of previously registered
	 *  listeners. It is not an error to remove a listener that was never
	 *  registered. A log listener may only be removed when the interpreter is
	 *  in the STOPPED sub-state.
	 *   @param pLogListener (in) Pointer to the listener to de-register.
	 *   @return VXM_RC_OK, VXM_RC_INVALID_STATE.
	 */
	virtual VxmRc removeLogListener(VxmLogListener *pLogListener) = 0;

	/** Process the current document. The run method can only be called in the
	 *  ALLOCATED state and STOPPED sub-state. Any other state will result in an
	 *  error. The setDocument method must be called prior to the run method.
	 *  The run method first checks to see which document to process by calling
	 *  the getDocument method. If there is no document to process, an error is
	 *  returned. The run method is a synchronous method call; it does not
	 *  return until the document is completely processed or the cancel method
	 *  is called or a fatal error occurs. If setUriResolver has not been
	 *  called, and the run method encounters an external reference, an error is
	 *  returned. The feature descriptor and property descriptor should not be
	 *  changed while the run method is processing. The run method may be called
	 *  repeatedly.
	 *   @return VXM_RC_OK, VXM_RC_MEM_ALLOC_ERROR, VXM_RC_INTERNAL_ERROR,
	 *    VXM_RC_INVALID_STATE, VXM_RC_NO_DOCUMENT, VXM_RC_NO_URI_RESOLVER,
	 *    VXM_RC_SEMANTIC_ERROR, VXM_RC_ECMASCRIPT_ERROR, VXM_RC_CANCELED.
	 */
	virtual VxmRc run() = 0;

	/** Process a handler. The runHandler method can only be called in the
	 *  ALLOCATED state and STOPPED sub-state. Any other state will result in an
	 *  error. Visual browsers must call addHandler before calling the
	 *  runHandler method. If the specified handler was added more than once,
	 *  the last one added is processed. If the specified handler was not added,
	 *  an error is returned. The runHandler method is a synchronous method
	 *  call; it does not return until the handler is completely processed or
	 *  the cancel method is called or a fatal error occurs. If setUriResolver
	 *  has not been called, and the runHandler method encounters an external
	 *  reference, an error is returned. The feature descriptor and property
	 *  descriptor should not be changed while the runHandler method is
	 *  processing. The runHandler method may be called repeatedly.
	 *   @param pstrHandlerId (in) Pointer to the "id" attribute of the handler
	 *    to process.
	 *   @param pstrDocUri (in) Pointer to the URI for the document containing
	 *    the handler.
	 *   @param bDebugMode (in) Set to true if browser is running in debug mode.
	 *   @return VXM_RC_OK, VXM_RC_MEM_ALLOC_ERROR, VXM_RC_INTERNAL_ERROR,
	 *    VXM_RC_INVALID_STATE, VXM_RC_HANDLER_NOT_FOUND,
	 *    VXM_RC_NO_URI_RESOLVER, VXM_RC_SEMANTIC_ERROR,
	 *    VXM_RC_ECMASCRIPT_ERROR, VXM_RC_CANCELED.
	 */
	virtual VxmRc runHandler(const wchar_t *pstrHandlerId, const wchar_t *pstrDocUri, bool bDebugMode = false) = 0;

	/**	Interrupt the current interpreter session and return to the STOPPED sub-
	 *  state. This method will change the interpreter sub-state to STOPPING
	 *  while the stopping operation completes. This method returns immediately.
	 *  Visual browsers should listen for the STOPPED sub-state change or wait
	 *  on the STOPPED sub-state to determine when the browser actually stops.
	 *   @return VXM_RC_OK, VXM_RC_MEM_ALLOC_ERROR, VXM_RC_INTERNAL_ERROR,
	 *    VXM_RC_INVALID_STATE.
	 */
	virtual VxmRc cancel() = 0;

	/** Start listening. This method should be called when the user wants to
	 *  start speaking, for example, when the user pushes the mic button. This
	 *  method should only be called while the interpreter is in the ALLOCATED
	 *  state, PROCESSING sub-state, and LISTENING_STOPPED listening state. If
	 *  the listening mode is push-to-activate, listening is automatically
	 *  stopped when silence is detected. If the listening mode is push-to-talk,
	 *  visual browsers should call the stopListening method when the user wants
	 *  to stop speaking.
	 *   @return VXM_RC_OK, VXM_RC_MEM_ALLOC_ERROR, VXM_RC_INTERNAL_ERROR,
	 *    VXM_RC_INVALID_STATE.
	 */
	virtual VxmRc startListening() = 0;

	/** Stop listening and process the audio input. This method should be called
	 *  when the user wants to stop speaking, such as when the user releases the
	 *  mic button. This method should only be called if the listening mode is
	 *  push-to-talk, and the interpreter is in the ALLOCATED state, PROCESSING
	 *  sub-state, and LISTENING_STARTED listening state. If the listening mode
	 *  is push-to-activate, listening is automatically stopped when silence is
	 *  detected.
	 *   @param bRecording (in) Recording flag. True to start recording, or
	 *    false to stop recording.
	 *   @return VXM_RC_OK, VXM_RC_MEM_ALLOC_ERROR, VXM_RC_INTERNAL_ERROR,
	 *    VXM_RC_INVALID_STATE, VXM_RC_INVALID_LISTENING_MODE.
	 */
	virtual VxmRc stopListening() = 0;

	/** Return the current interpreter state, sub-state, and listening state.
	 *   @return The interpreter's current state.
	 */
	virtual long getState() = 0;

	/** Get a pointer to the audio data for a recording made by the
	 *  interpreter while visiting a VoiceXML <record> element.
	 *   @param pstrRecHandle (in) Handle to the recording. This is the string
	 *    value of the "handle" property of an ECMAScript object of type
	 *    VXMLAudioRecording.
	 *   @param pstrFormat (out) MIME content type of the recording. This should
	 *    be used when building multipart/form-data. 
	 *   @param ppBuffers (out) Pointer to an array of VxmAudioBuffer structs.
	 *   @param puiCount (out) Number of elements in *ppBuffers array.
	 *   @return VXM_RC_OK, VXM_RC_INTERNAL_ERROR, VXM_RC_INVALID_REC_HANDLE.
	 */
	virtual VxmRc getRecording(const wchar_t *pstrRecHandle, wchar_t **pstrFormat, VxmAudioBuffer **ppBuffers, unsigned int *puiCount) = 0;

	/** Release the audio data for a recording made by the interpreter while
	 *  visiting a VoiceXML <record> element.
	 *   @param pstrRecHandle (in) Handle to the recording. This is the string
	 *    value of the "handle" property of an ECMAScript object of type
	 *    VXMLAudioRecording.
	 *   @return VXM_RC_OK, VXM_RC_INTERNAL_ERROR, VXM_RC_INVALID_REC_HANDLE.
	 */
	virtual VxmRc releaseRecording(const wchar_t *pstrRecHandle) = 0;

	/** Get the create scope callback function
	 *   @return Pointer to the CreateScopeCB function
	 */
	virtual CreateScopeCB * getScopeFactory() = 0;

	/** Set the create scope callback function
	 *   @param pScopeFactory (in) Pointer to the scope callback function
	 *   @return void
	 */
	virtual void setScopeFactory(CreateScopeCB *pScopeFactory) = 0;

	/** Add a voiceXML link to the list of links for this interpreter session.
	  *  Each link is treated as a VoiceXML document <link> for all the handlers
	  *  added in this interpreter session. A <link> may only be added when the interpreter
	  *  is in the STOPPED sub-state.
	  *   @param pstrLinkText (in) Pointer to the text of a VoiceXML <link> fragment.
	  *   @param pstrDocUri (in) Pointer to the URI for the document associated with this link.
	  *   @param lDocTimeStamp (in) The document timestamp. Format is not important,
	  *    as long as the timestamp changes each time the document is refreshed.
	  *   @param pstrBaseUri (in) Pointer to the base URI for resolving external
	  *    specified by the markup, pstrBaseUri should be the same as pstrUri.
	  *   @return VXM_RC_OK, VXM_RC_MEM_ALLOC_ERROR, VXM_RC_INTERNAL_ERROR,
	  *    VXM_RC_INVALID_STATE, VXM_PARSE_FAILED.
	  */
	 virtual VxmRc addLink(const wchar_t *pstrLinkText, const wchar_t *pstrDocUri, long lDocTimeStamp, const wchar_t *pstrBaseUri) = 0;

	/** Add a sync tag as specified in the document.  Sync tags are added as they
	 *  are parsed by the visual browser.
	 *  @param  pstrbaseURI (in) The base URI of the VoiceXML Form containing the Field.
	 *  @param  pstrCurDocURI (in) The URI of the current X+V document.
	 *  @param  pstrHtmlFormId (in) The HTML form's id attribute.  NULL should be 
	 *                           entered if the html-form-id attribute is missing.
	 *  @param  pstrFieldId (in) The sync tag's Field attribute.  The Field attribute
	 *                           specifies the xv:id attribute of a VoiceXML field element.
	 *  @param  pstrHtmlName (in) The sync tag's Name attribute.  the Name attribute
	 *                           specifies an HTML form control name or control group name.
	 *  @return VXM_RC_OK, VXM_RC_MEM_ALLOC_ERROR, VXM_RC_INTERNAL_ERROR,
	 *    VXM_RC_INVALID_STATE
	*/
	virtual VxmRc addSync(const wchar_t* baseURI, const wchar_t* pstrCurDocURI, const wchar_t* pstrHtmlFormId, const wchar_t* pstrFieldId, const wchar_t* pstrHtmlName) = 0;

	/** Notify the active voice handler that the HTML control has been updated with
	 *  the mouse or keyboard.  Notification occurs when the HTML control that is
	 *  synchronized loses the input focus.  This corresponds to the "blur" event.
	 *  @param  pstrHtmlFormId (in) The HTML form's id attribute.  NULL should be 
	 *                       entered if the html-form-id attribute is missing.
	 *  @param  pstrHtmlName (in) The sync tag's Name attribute.  the Name attribute
	 *                       specifies an HTML form control name or control group name.
	 *  @return VXM_RC_OK, VXM_RC_MEM_ALLOC_ERROR, VXM_RC_INTERNAL_ERROR,
	 *    VXM_RC_INVALID_STATE, VXM_NO_SYNC_FIELD.
	 */
	virtual VxmRc syncBlur(const wchar_t* pstrHtmlFormId, const wchar_t* pstrHtmlName) = 0;

	/** Notify the active voice handler that an HTML control has been selected.
	 *  Notification occurs when the HTML control that is synchronized gains the input
	 *  focus.  This corresponds to the "focus" event.
	 *  @param  pstrHtmlFormId (in) The HTML form's id attribute.  NULL should be 
	 *                       entered if the html-form-id attribute is missing.
	 *  @param  pstrHtmlName (in) The sync tag's Name attribute.  the Name attribute
	 *                       specifies an HTML form control name or control group name.
	 *  @return VXM_RC_OK, VXM_RC_MEM_ALLOC_ERROR, VXM_RC_INTERNAL_ERROR,
	 *    VXM_RC_INVALID_STATE, VXM_NO_SYNC_FIELD.
	 */
	virtual VxmRc syncFocus(const wchar_t* pstrHtmlFormId, const wchar_t* pstrHtmlName) = 0;

	/** Interrupt the specific interpreter session. If the specified handler is not 
	 *  currently running, this method returns immediately with VXM_RC_NOT_RUNNING. 
	 *
	 *  If the specified handler is currently running, it interrupts the interpreter session and 
	 *  returns to the STOPPED sub-state. This method will change the interpreter sub-state
	 *  to STOPPING while the stopping operation completes. This method returns immediately. 
	 *  Visual browsers should listen for the STOPPED sub-state change or wait on the STOPPED 
	 *  sub-state to determine when the browser actually stops.
	 *
	 *   @param pstrHandlerId (in) Pointer to the "id" attribute of the handler
	 *    to be canceled.
	 *   @return VXM_RC_OK, VXM_RC_NOT_RUNNING, VXM_RC_MEM_ALLOC_ERROR, VXM_RC_INTERNAL_ERROR,
	 *    VXM_RC_INVALID_STATE.
	 */
	virtual VxmRc cancel(const wchar_t *pstrHandlerId) = 0;

	/** Set the maximum number of VxmInterpreter instance. The number can only be set before creating 
	 *   the first VxmInterpreter instance and is sealed afterwards. It can be set again after all the instances have been deleted.
	 *   The VxmInterpreter::createInstance will return NULL if the number of instances excesses the maximum number. 
	 *
	 *   @param iMaxInstances (in) The maximum number of VxmInterpreter instances. The number must be greater than 0.
	 *   @return VXM_RC_OK, VXM_RC_SEALED, VXM_RC_INVALID_VALUE
	 */
	static VxmRc setMaxInstances(int iMaxInstances);

	/** Gets the maximum number of VxmInterpreter instance.
	 *   @return The maximum number of VxmInterpreter instances.
	 */
	static int getMaxInstances();

	/** Return a string containing the version and build date to be
	 *  used when displaying the browser's about box
	 *   @return A pointer to a Unicode string
	 */
	static const wchar_t * getVersionString();

	/** Return a string containing the build number to be
	 *  used when displaying the browser's about box
	 *   @return A pointer to a Unicode string
	 */
	static const wchar_t * getBuildString();

	/** Return a string containing the user-agent information 
	 *  used in User-Agent request-header as a part of comment  parenthesis.
	 *  For example,
	 *  Mozilla/4.0 (compatible; MSIE, 6.0) BrowserName 1.0 (IBM Embedded
	 *  ViaVoice/3.0/ EAK01AG9/LE)
	 *  
	 *   @return A pointer to a Unicode string
	 */
	static const wchar_t * getUserAgentString();

};

#endif // #ifndef VXM_H
