/****************************************************************/
/* IBM Confidential OCO Source Materials                        */
/* AS854ZZ                                                      */
/* (C) Copyright IBM Corp. 2003                                 */
/*                                                              */
/* The source code for this program is not published or other-  */
/* wise divested of its trade secrets, irrespective of what has */
/* been deposited with the U.S. Copyright Office.               */
/****************************************************************/

#ifndef VXMDEFS_H
#define VXMDEFS_H

/** Audio buffer.
 */
struct VxmAudioBuffer
{
	/** Audio buffer.
	 */
	void *pBuf;

	/** Number of bytes in the audio buffer.
	 */
	size_t size;
};

/** Uri errors.
 */
enum VxmUriError
{
	VXM_URI_ERROR_BADFETCH,
	VXM_URI_ERROR_TIMEOUT,
	VXM_URI_ERROR_SEMANTIC
};

/** Class VxmUriListener is an abstract base class that provides a listener
 *  interface for results from VxmUriResolver::vxmResolveTextUri and
 *  VxmUriResolver::vxmResolveBinaryUri.
 */
class VxmUriListener
{
public:
	/** Process text data resulting from a call to
	 *  VxmUriResolver::vxmResolveTextUri.
	 *   @param pstrUri (in) Pointer to the URI.
	 *   @param pstrText (in) Pointer to the text data.
	 *   @param pstrMimeType (in) Pointer to the mime type of the text data.
	 *   @param lTimeStamp (in) The document timestamp. Format is not important,
	 *    as long as the timestamp changes each time the document is refreshed.
	 *   @param uriListenerData (in) Pointer passed into
	 *    VxmUriResolver::vxmResolveTextUri.
	 */
	virtual void processTextUri(const wchar_t *pstrUri, const wchar_t *pstrText, const wchar_t *pstrMimeType, long lTimeStamp, void *uriListenerData) = 0;

	/** Process binary data resulting from a call to
	 *  VxmUriResolver::vxmResolveBinaryUri.
	 *   @param pstrUri (in) Pointer to the URI.
	 *   @param pucBuf (in) Pointer to the binary data buffer.
	 *   @param iBufSize (in) Buffer size, in bytes.
	 *   @param bFinalBuf (in) True if this is the final binary data buffer, or
	 *    false otherwise.
	 *   @param pstrMimeType (in) Pointer to the mime type of the text data.
	 *   @param lTimeStamp (in) The document timestamp. Format is not important,
	 *    as long as the timestamp changes each time the document is refreshed.
	 *   @param uriListenerData (in) Pointer passed into
	 *    VxmUriResolver::vxmResolveBinaryUri.
	 */
	virtual void processBinaryUri(const wchar_t *pstrUri, unsigned char *pucBuf, int iBufSize, bool bFinalBuf, const wchar_t *pstrMimeType, long lTimeStamp, void *uriListenerData) = 0;

	/** Process prompt text data resulting from a call to
	 *  VxmUriResolver::vxmResolvePromptTextUri.
	 *   @param pstrUri (in) Pointer to the URI.
	 *   @param pstrText (in) Pointer to the text data.
	 *   @param pstrMimeType (in) Pointer to the mime type of the text data.
	 *   @param lTimeStamp (in) The document timestamp. Format is not important,
	 *    as long as the timestamp changes each time the document is refreshed.
	 *   @param uriListenerData (in) Pointer passed into
	 *    VxmUriResolver::vxmResolvePromptTextUri.
	 *   @param pstrAcssProperties (in) array of wchar string that contains only voice
	 *    styling properties, such as "voice-family: male".
	 *   @param iAcssPropertySize (in) the number of  properties.
	 *   @param bFinalMessage (in) true if this is the final message, false otherwise.
	 */
	virtual void processPromptTextUri(const wchar_t *pstrUri, const wchar_t *pstrText, const wchar_t **pstrAcssProperties, int iAcssPropertySize, bool bFinalMessage, const wchar_t *pstrMimeType, long lTimeStamp, void *uriListenerData) = 0;

	/** Process an error resulting from a call to either
	 *  VxmUriResolver::vxmResolveTextUri or
	 *  VxmUriResolver::vxmResolveBinaryUri.
	 *   @param pstrUri (in) Pointer to the URI.
	 *   @param error (in) Error.
	 *   @param pstrResponseCode (in) Pointer to the protocol response code in
	 *    the case of a VXM_URI_ERROR_BADFETCH, or NULL otherwise.
	 *   @param uriListenerData (in) Pointer passed into
	 *    VxmUriResolver::vxmResolveTextUri or
	 *    VxmUriResolver::vxmResolvePromptTextUri or
	 *    VxmUriResolver::vxmResolveBinaryUri.
	 */
	virtual void processUriError(const wchar_t *pstrUri, VxmUriError error, const wchar_t *pstrResponseCode, void *uriListenerData) = 0;

	/** Extract the charset from the first line of text data resulting from a
	 *  call to VxmUriResolver::vxmResolveTextUri.
	 *   @param pstrHeader (in) First line of the text data.
	 *   @param pstrMimeType (in) Pointer to the mime type of the text data.
	 *   @param uriListenerData (in) Pointer passed into
	 *    VxmUriResolver::vxmResolveTextUri.
	 *   @return The charset, or NULL if the header is invalid or the mime type
	 *    is not supported, or empty string if there is no charset specified in the header.
	 *    When empty string is returned, the caller should use its own default charset. 
	 */
	virtual wchar_t* uriHeaderToCharset(const wchar_t *pstrHeader, const wchar_t *pstrMimeType, void *uriListenerData) = 0;
};

/** Class VxmUriResolver is an abstract base class that provides an interface
 *  for resolving URIs on behalf of a VxmInterpreter instance.
 */
class VxmUriResolver
{
public:
	/** Resolve a text URI asynchronously. When the text is ready, it is sent to
	 *  a VxmUriListener.
	 *   @param pstrUri (in) Pointer to the URI.
	 *   @param pstrBaseUri (in) Pointer to the base URI for resolving this URI.
	 *   @param bXml (in) True if the requested document is XML, or false.
	 *    otherwise. If the requested document is not XML, the charset can be
	 *    determined by calling VxmUriListener::uriHeaderToCharset.
	 *   @param bUrgent (in) True if the end-user is waiting for the URI to be
	 *    resolved, or false otherwise.
	 *   @param lTimeout (in) Timeout in milliseconds. If lTimeout is negative,
	 *    there is no timeout.
	 *   @param lMaxAge (in) Maximum age of the content, in milliseconds.
	 *    However, the content must not have expired, unless lMaxStale is
	 *    greater than 0. If lMaxAge is negative, the document is willing to use
	 *    unexpired content of any age.
	 *   @param ulMaxStale (in) Maximum staleness of the content, in
	 *    milliseconds. The content may have exceeded its expiration time by no
	 *    more than the specified number of milliseconds. If lMaxStale is 0, the
	 *    content must not have expired.
	 *   @param uriListener (in) Pointer to the listener to be notified once the
	 *    URI has been resolved.
	 *   @param uriListenerData (in) Pointer to be passed back to uriListener.
	 *   @return ID that can be passed to vxmCancelResolveUri.
	 */
	virtual long vxmResolveTextUri(const wchar_t *pstrUri, const wchar_t *pstrBaseUri, bool bXml, bool bUrgent, long lTimeout, long lMaxAge, unsigned long lMaxStale, VxmUriListener *pUriListener, void *pUriListenerData) = 0;

	/** Resolve a prompt text URI asynchronously. When the text is ready, it is sent to
	 *  a VxmUriListener.
	 *   @param pstrUri (in) Pointer to the URI.
	 *   @param pstrBaseUri (in) Pointer to the base URI for resolving this URI.
	 *   @param bXml (in) True if the requested document is XML, or false.
	 *    otherwise. If the requested document is not XML, the charset can be
	 *    determined by calling VxmUriListener::uriHeaderToCharset.
	 *   @param bUrgent (in) True if the end-user is waiting for the URI to be
	 *    resolved, or false otherwise.
	 *   @param lTimeout (in) Timeout in milliseconds. If lTimeout is negative,
	 *    there is no timeout.
	 *   @param lMaxAge (in) Maximum age of the content, in milliseconds.
	 *    However, the content must not have expired, unless lMaxStale is
	 *    greater than 0. If lMaxAge is negative, the document is willing to use
	 *    unexpired content of any age.
	 *   @param ulMaxStale (in) Maximum staleness of the content, in
	 *    milliseconds. The content may have exceeded its expiration time by no
	 *    more than the specified number of milliseconds. If lMaxStale is 0, the
	 *    content must not have expired.
	 *   @param uriListener (in) Pointer to the listener to be notified once the
	 *    URI has been resolved.
	 *   @param uriListenerData (in) Pointer to be passed back to uriListener.
	 *   @return ID that can be passed to vxmCancelResolveUri.
	 */
	virtual long vxmResolvePromptTextUri(const wchar_t *pstrUri, const wchar_t *pstrBaseUri, bool bXml, bool bUrgent, long lTimeout, long lMaxAge, unsigned long lMaxStale, VxmUriListener *pUriListener, void *pUriListenerData) = 0;

	/** Resolve a binary URI asynchronously. When each binary buffer is ready,
	 *  it is sent to a VxmUriListener.
	 *   @param pstrUri (in) Pointer to the URI.
	 *   @param pstrBaseUri (in) Pointer to the base URI for resolving this URI.
	 *   @param iBufSize (in) Preferred buffer size, in bytes.
	 *   @param bUrgent (in) True if the end-user is waiting for the URI to be
	 *    resolved, or false otherwise.
	 *   @param lTimeout (in) Timeout in milliseconds. If lTimeout is negative,
	 *    there is no timeout.
	 *   @param lMaxAge (in) Maximum age of the content, in milliseconds.
	 *    However, the content must not have expired, unless lMaxStale is
	 *    greater than 0. If lMaxAge is negative, the document is willing to use
	 *    unexpired content of any age.
	 *   @param ulMaxStale (in) Maximum staleness of the content, in
	 *    milliseconds. The content may have exceeded its expiration time by no
	 *    more than the specified number of milliseconds. If lMaxStale is 0, the
	 *    content must not have expired.
	 *   @param uriListener (in) Pointer to the listener to be notified once the
	 *    URI has been resolved.
	 *   @param uriListenerData (in) Pointer to be passed back to uriListener.
	 *   @return ID that can be passed to vxmCancelResolveUri.
	 */
	virtual long vxmResolveBinaryUri(const wchar_t *pstrUri, const wchar_t *pstrBaseUri, int iBufSize, bool bUrgent, long lTimeout, long lMaxAge, unsigned long lMaxStale, VxmUriListener *pUriListener, void *pUriListenerData) = 0;

	/** Cancel a previous request to vxmResolveTextUri or vxmResolveBinaryUri.
	 *   @param pId (in) ID of the request to be cancelled, as returned from
	 *    vxmResolveTextUri or vxmResolveBinaryUri. If NULL, all requests are
	 *    cancelled.
	 */
	virtual void vxmCancelResolveUri(void *pId) = 0;

	/** Prefetch a text URI asynchronously. Resulting text data should be cached
	 *  as long as possible, in order to support future calls to
	 *  vxmResolveTextUri.
	 *   @param pstrUri (in) Pointer to the URI.
	 *   @param pstrBaseUri (in) Pointer to the base URI for resolving this URI.
	 *   @param bXml (in) True if the requested document is XML, or false.
	 *    otherwise. If the requested document is not XML, the charset can be
	 *    determined by calling VxmUriListener::uriHeaderToCharset.
	 */
	virtual void vxmPrefetchTextUri(const wchar_t *pstrUri, const wchar_t *pstrBaseUri, bool bXml) = 0;

	/** Prefetch a prompt text URI asynchronously. Resulting text data should be cached
	 *  as long as possible, in order to support future calls to
	 *  vxmResolveTextUri.
	 *   @param pstrUri (in) Pointer to the URI.
	 *   @param pstrBaseUri (in) Pointer to the base URI for resolving this URI.
	 *   @param bXml (in) True if the requested document is XML, or false.
	 *    otherwise. If the requested document is not XML, the charset can be
	 *    determined by calling VxmUriListener::uriHeaderToCharset.
	 */
	virtual void vxmPrefetchPromptTextUri(const wchar_t *pstrUri, const wchar_t *pstrBaseUri, bool bXml) = 0;

	/** Prefetch a binary URI asynchronously. Resulting binary data should be
	 *  cached as long as possible, in order to support future calls to
	 *  vxmResolveBinaryUri.
	 *   @param pstrUri (in) Pointer to the URI.
	 *   @param pstrBaseUri (in) Pointer to the base URI for resolving this URI.
	 *   @param iBufSize (in) Preferred buffer size, in bytes.
	 */
	virtual void vxmPrefetchBinaryUri(const wchar_t *pstrUri, const wchar_t *pstrBaseUri, int iBufSize) = 0;

	/** Cancel all previous requests to vxmPrefetchTextUri and
	 *  vxmPrefetchBinaryUri.
	 */
	virtual void vxmCancelPrefetches() = 0;
};

#endif // #ifndef VXMDEFS_H
