#if !defined(SCOPE_H)
#define SCOPE_H

#include "vxm.h"

#pragma pack(4)

// Handle to EcmaScript object
typedef void* ESObject;

/*
 * EcmaScript Types
 */
enum EcmaScriptValueType
{
    SCOPE_VALUE_UNDEFINED,
    SCOPE_VALUE_NULL,
    SCOPE_VALUE_BOOLEAN,
    SCOPE_VALUE_NUMBER,
    SCOPE_VALUE_STRING,
    SCOPE_VALUE_OBJECT
};

/*
 * Result type for all methods requiring a result
 */
struct EcmaScriptValue
{
  EcmaScriptValueType type;
  struct value_struct
  {
    double num_val;
    wchar_t * str_val;
    ESObject object_val;
    bool bool_val;
  } value;
};

/**Class Scope implements an EcmaScript object scope chain and encapsulates
 * the Ecmascript interpreter. Scope is an abstract class.  
 */
class VxmScope
{
public:
	/**Ctor
	 */
	VxmScope() {}
    /**Dtor
     */
	virtual ~VxmScope() {}
    /**Evaluate the EcmaScript contained in script as program text.
     *  Return the result of the evaluation in the result.
     * @param script (in) Program text to compile and execute.
     * @param result (out) Result of script evaluation.  Input to result
	 *                     is NULL if caller not interested in it.
	 * @param bReleaseObject (in) if true release object if object returned
	 * @return VXM_RC_OK, VXM_RC_INTERNAL_ERROR, VXM_RC_CANCELED
     */
	virtual VxmRc eval(const wchar_t * script, EcmaScriptValue * result, bool bReleaseObject) = 0;
    /**Enter a new inner VxmScope
     * @param name (in) Name of the scope object
	 * @param object (out) scope object created by enter.  Input to object
	 *                     is NULL if caller not interested in it.
	 * @return VXM_RC_OK, VXM_RC_INTERNAL_ERROR, VXM_RC_CANCELED
     */
	virtual VxmRc enter(const wchar_t * name, ESObject* object) = 0;
    /**Leave the current scope
	 * @return VXM_RC_OK, VXM_RC_INTERNAL_ERROR, VXM_SCOPE_RC_NO_OBJECT
     */
	virtual VxmRc leave() = 0;
	/**Create an unnamed object
	 * @param object (out) handle to the created object.  This
	 *                  object must be later released by
	 *                  calling releaseObject
	 * @return VXM_RC_OK, VXM_RC_INTERNAL_ERROR, VXM_RC_CANCELED,
	 *         VXM_SCOPE_RC_NO_OBJECT
	 */
	virtual VxmRc createObject(ESObject* object) = 0;
	/**Create an unnamed array
	 * @param array (out) handle to the created array.  This
	 *                    array must be later released by
	 *                    calling releaseObject.
	 * @return VXM_RC_OK, VXM_RC_INTERNAL_ERROR, VXM_RC_CANCELED,
	 *         VXM_SCOPE_RC_NO_OBJECT
	 */
	virtual VxmRc createArrayObject(ESObject* array) = 0;
	/**Release an object or array
	 * @param object (in) handle to the object
	 * @return VXM_RC_OK, VXM_RC_INTERNAL_ERROR, VXM_SCOPE_RC_NO_OBJECT
	 */
	virtual VxmRc releaseObject(ESObject object) = 0;
    /**Get the value of a field within an object
     * @param slotName (in) Name of the field
     * @param result (out) returned value of the field
	 * @return VXM_RC_OK, VXM_RC_INTERNAL_ERROR, VXM_RC_CANCELED,
	 *         VXM_SCOPE_RC_NO_OBJECT
     */
    virtual VxmRc getSlot(const wchar_t * slotName, EcmaScriptValue* result) = 0;
    /**Get the value of a field within an object
     * @param slotName (in) Name of the field
     * @param result (out) returned value of the field
	 * @param object (in) handle to the slot's object.
	 * @return VXM_RC_OK, VXM_RC_INTERNAL_ERROR, VXM_RC_CANCELED
     */
    virtual VxmRc getSlot(const wchar_t * slotName, EcmaScriptValue* result, ESObject object) = 0;
    /**Get the value of an element within an array
     * @param index (in) index number of the element in array
     * @param result (out) returned value of the field
	 * @param array (in) handle to the element's array
	 * @return VXM_RC_OK, VXM_RC_INTERNAL_ERROR, VXM_RC_CANCELED,
	 *         VXM_SCOPE_RC_NO_OBJECT
     */
    virtual VxmRc getElement(int index, EcmaScriptValue* result, ESObject array) = 0;
	/**Get the recording value of a field within an object
     * @param value (in) the audio recording container
	 * @param recHandle (out) returned recording handle of the field
	 * @return VXM_RC_OK, VXM_RC_INTERNAL_ERROR, VXM_RC_CANCELED,
	 *         VXM_RC_INVALID_REC_HANDLE
	 */
	virtual VxmRc getRecording(const EcmaScriptValue& value, wchar_t ** recHandle) = 0;
    /**Set the value of a field within an object
     * @param slotName (in) name of the field
     * @param value (in) the new value of the field
	 * @return VXM_RC_OK, VXM_RC_INTERNAL_ERROR, VXM_RC_CANCELED,
	 *         VXM_SCOPE_RC_NO_OBJECT
     */
    virtual VxmRc setSlot(const wchar_t * slotName, const EcmaScriptValue& value) = 0;
    /**Set the value of a field within an object
     * @param slotName (in) name of the field
     * @param value (in) the new value of the field
	 * @param object (in) handle to the parent object.
	 * @return VXM_RC_OK, VXM_RC_INTERNAL_ERROR, VXM_RC_CANCELED
     */
    virtual VxmRc setSlot(const wchar_t * slotName, const EcmaScriptValue& value, ESObject object) = 0;
    /**Set the value of a field within an object
     * @param index (in) index number of the element in array
     * @param value (in) The new value of the field
	 * @param array (in) handle to the element's array
	 * @return VXM_RC_OK, VXM_RC_INTERNAL_ERROR, VXM_RC_CANCELED,
	 *         VXM_SCOPE_RC_NO_OBJECT
     */
    virtual VxmRc setElement(int index, const EcmaScriptValue& value, ESObject array) = 0;
	/**Set the value of a field within an object to be a handle to a recording
	 * @param slotName (in) name of the field
	 * @param recHandle (in) returned recording handle of the field
	 * @return VXM_RC_OK, VXM_RC_INTERNAL_ERROR, VXM_RC_CANCELED
	 */
	virtual VxmRc setRecording(const wchar_t * slotName, const wchar_t * recHandle) = 0;
	/**Delete a string that was created from a previous eval(), getSlot() or getElement() call
	 * @param ptrString (in) pointer to the string to free
	 * @return VXM_RC_OK, VXM_RC_INTERNAL_ERROR, VXM_RC_CANCELED
	 */
	virtual VxmRc freeString(wchar_t * ptrString) = 0;

};

#pragma pack()

#endif //!defined(SCOPE_H)
